/*
  capstats - capstats.h
  Copyright (C) 2009 Bert Vermeulen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CAPSTATS_H_
#define CAPSTATS_H_

#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>

#include <pcap.h>


/*
 * constants
 */
#define PROGNAME		"capstats"
#define CMDSOCKET		"/var/run/capstats"
#define CLIENTSOCKET	"/tmp/capstats-"
#define PINGSOCKET		"/tmp/capstats-ping-"
#define PERMDIR		"/var/lib/capstats"
#define STATSFILE		PERMDIR "/stats"
#define SESSIONSFILE	PERMDIR "/sessions"

#define MAX_SESSIONS 20
#define SNAPLEN 64
#define CAPDELAY 1000
#define OFFSET_ETH_PAYLOAD 14
#define OFFSET_ETH_ETHERTYPE 12
#define OFFSET_IP_SRCADDR 12
#define MAX_TOKENS 20
#define MAX_SESSION_NAME_LENGTH 32
#define MAX_FILTER_LEN 256
#define KEEPALIVE_TIMEOUT 2
#define KEEPALIVE_INTERVAL 5
#define MAX_LOGMSG_LEN 255

#define MSG_PERMDENIED "permission denied" "\nNOK\n"

/*
 * structs
 */
struct capsession {
	struct capsession *next;
	/* session info */
	char name[MAX_SESSION_NAME_LENGTH];
	char *filter;
	pcap_t *p;
	int linktype;
	int payload_offset;
	int fd;
	time_t starttime;
	/* stats */
	uint pkts;
	uint pkts_in;
	uint pkts_out;
	uint bytes_in;
	uint bytes_out;
};


struct pollevent {
	struct pollevent *next;
	struct capsession *session;
	time_t polltime;
	struct sockaddr *sa;
	socklen_t sa_len;
	int frequency;
};


/*
 * prototypes
 */
int send_command(char *command);
int start_daemon();
char **tokenize(char *cmd);
int cmd_ping(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_new(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_modify(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_stop(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_list(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_show(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_poll(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
int cmd_watch(const char **tokens, struct sockaddr *sa, socklen_t sa_len, char **errmsg);
void proc_packet(struct capsession *session, const u_char *pkt, int pkt_len);
int init_session(char *session_name, char *filter, char **errmsg);
int stop_session(struct capsession *session, char **errmsg);
void stop_all_sessions(void);
void kill_daemon(int signum);
void load_sessions(void);
void save_sessions(void);
void load_session_stats(struct capsession *session);
void save_session_stats(void);


#endif /* CAPSTATS_H_ */
