/*
  capstats - main.c
  Copyright (C) 2009 Bert Vermeulen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <grp.h>

#include "capstats.h"

#define MODE_CLIENT 0
#define MODE_DAEMON 1

/*
 * globals
 */
int debug = 0;
int client_gid = -1;


void usage()
{

	printf("usage: capstats [-D] -d [-g gid]\n");
	printf("usage: capstats [-D] <command>\n");
	exit(1);

}


gid_t get_gid_from_arg(char *arg)
{
	struct group *grp;
	gid_t gid;
	int is_nonnum, i;

	gid = -1;
	is_nonnum = 0;
	for(i = 0; arg[i]; i++)
	{
		if(arg[i] < '0' || arg[i] > '9')
		{
			is_nonnum = 1;
			break;
		}
	}

	if(is_nonnum)
	{
		errno = 0;
		grp = getgrnam(arg);
		if(grp)
			gid = grp->gr_gid;
		else
			printf("%s\n", strerror(errno));
	}
	else
		gid = atoi(arg);

	return gid;
}


int main(int argc, char **argv)
{
	int mode, cnt, ret, i, j;
	char *command;

	ret = 0;
	command = NULL;
	mode = MODE_CLIENT;
	for(i = 1; i < argc; i++)
	{
		if(argv[i][0] == '-')
		{
			if(!strcmp(argv[i], "-d"))
				mode = MODE_DAEMON;
			else if(!strcmp(argv[i], "-D"))
				debug = 1;
			else if(!strcmp(argv[i], "-g"))
			{
				if(i+1 < argc)
				{
					client_gid = get_gid_from_arg(argv[++i]);
					if(client_gid == -1)
					{
						/* invalid gid specified was already reported */
						exit(1);
					}
					if(debug)
						printf("access restricted to gid %d\n", client_gid);
				}
			}
			else
				usage();
		}
		else if(mode == MODE_CLIENT)
		{
			/* convert the rest of the args into a command string */
			/* spaces between args + terminating zero from the fencepost */
			cnt = argc - i;
			for(j = i; j < argc; j++)
			{
				cnt += strlen(argv[j]);
			}
			command = malloc(cnt);
			command[0] = '\0';
			for(j = i; j < argc; j++)
			{
				strcat(command, argv[j]);
				strcat(command, " ");
			}
			command[strlen(command)-1] = 0;
			break;
		}
		else
			usage();
	}

	if(mode == MODE_CLIENT)
	{
		if(command)
			ret = send_command(command);
		else
			usage();
	}
	else
	{
		if(debug || fork() == 0)
			ret = start_daemon();
	}

	return(ret);
}

