/*
  capstats - parser.c
  Copyright (C) 2009 Bert Vermeulen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <wctype.h>


#include "capstats.h"

#define MODE_START 1
#define MODE_TOKEN 2
#define MODE_QUOTED 3

extern int debug;


char **tokenize(char *cmd)
{
	int start, mode, token_start, num_tokens, i;
	char **tokens;
	u_char *buf;

	tokens = malloc(sizeof(char *) * (MAX_TOKENS+1));
	buf = malloc(strlen(cmd) + 2);
	/* strip leading whitespace */
	for(start = 0; iswspace(cmd[start]); start++);
	strcpy((char *) buf, cmd + start);
	/* extra space to make sure tokenizing finishes properly */
	strcat((char *) buf, " ");

	mode = MODE_START;
	num_tokens = 0;
	token_start = 0;
	for(i = 0; buf[i] != 0; i++)
	{
		if(num_tokens == MAX_TOKENS)
		{
			printf("too many tokens in command\n");
			free(buf);
			free(tokens);
			return(NULL);
		}

		if(mode == MODE_START)
		{
			if(buf[i] == '"')
			{
				mode = MODE_QUOTED;
				token_start = i + 1;
			}
			else if(!iswspace(buf[i]))
			{
				/* skip extra whitespace between tokens */
				mode = MODE_TOKEN;
				token_start = i;
			}
		}
		else if(mode == MODE_QUOTED)
		{
			if(buf[i] == '"')
			{
				/* closing quote */
				mode = MODE_START;
				buf[i] = '\0';
				tokens[num_tokens++] = (char *) buf + token_start;
				token_start = 0;
			}
		}
		else if(mode == MODE_TOKEN)
		{
			if(iswspace(buf[i]))
			{
				buf[i] = '\0';
				tokens[num_tokens++] = (char *) buf + token_start;
				token_start = 0;
				mode = MODE_START;
			}
		}
	}
	tokens[num_tokens] = NULL;

	if(debug)
		for(i = 0; i < num_tokens; i++)
			printf("token %d: [%s]\n", i, tokens[i]);

	if(num_tokens == 0)
	{
		free(buf);
		free(tokens);
		return(NULL);
	}

	return(tokens);
}
